/* Target-specific functions for making Whetstone work on the STM8
   Author: Philipp Klaus Krause */

#include <stdio.h>
#include <stdint.h>

#include <stdio.h>
#include <stdint.h>

#define MSC_IOMXP0	(*(volatile uint8_t *)0x503a)

#define CLK_CKDIVR	(*(volatile uint8_t *)0x50c6)
#define CLK_PCKENR1	(*(volatile uint8_t *)0x50c7)

#define STMR_CR1	(*(volatile uint8_t *)0x5340)
#define STMR_IER	(*(volatile uint8_t *)0x5341)
#define STMR_SR1	(*(volatile uint8_t *)0x5342)
#define STMR_EGR	(*(volatile uint8_t *)0x5343)
#define STMR_CNTH	(*(volatile uint8_t *)0x5344)
#define STMR_CNTL	(*(volatile uint8_t *)0x5345)
#define STMR_PSCL	(*(volatile uint8_t *)0x5346)
#define STMR_ARRH	(*(volatile uint8_t *)0x5347)
#define STMR_ARRL	(*(volatile uint8_t *)0x5348)

#define UART_SR	(*(volatile uint8_t *)0x5230)
#define UART_DR	(*(volatile uint8_t *)0x5231)
#define UART_BRR1	(*(volatile uint8_t *)0x5232)
#define UART_BRR2	(*(volatile uint8_t *)0x5233)
#define UART_CR2	(*(volatile uint8_t *)0x5235)
#define UART_CR3	(*(volatile uint8_t *)0x5236)

#define UART_CR2_TEN (1 << 3)
#define UART_CR3_STOP2 (1 << 5)
#define UART_CR3_STOP1 (1 << 4)
#define UART_SR_TXE (1 << 7)

void init(void)
{
	CLK_CKDIVR = 0x00; // Set the frequency to 16 MHz

	// Configure timer
	STMR_PSCL = 0x07;   // Timer frequency 125 kHz (lowest possible).
	STMR_ARRH = 0;
	STMR_ARRL = 125;    // Event frequency 1 kHz.
	STMR_EGR = 0x01;    // Re-initialize timer.
	STMR_CR1 = 0x05;    // Start timer.
	STMR_IER = 0x01;    // Enable timer interrupts.

	CLK_PCKENR1 = 0xff; // Enable peripherals

	MSC_IOMXP0 = 0x01;  // Use pins 22, 23 for UART instead of GPIO.

	UART_CR2 = UART_CR2_TEN; // Allow TX & RX
	UART_CR3 &= ~(UART_CR3_STOP1 | UART_CR3_STOP2); // 1 stop bit
	UART_BRR2 = 0x03; UART_BRR1 = 0x68; // 9600 baud

	__asm;
	rim                 // Enable interrupts.
	__endasm;
}

static volatile unsigned long c;

unsigned long clock(void)
{
	return(c);
}

void stmr(void) __interrupt(23)
{
	c++;
	STMR_SR1 = 0x00;
}

#if __SDCC_REVISION >= 9624
int putchar(int c)
{
	while(!(UART_SR & UART_SR_TXE));

	UART_DR = c;

	return(c);
}
#else
void putchar(char c)
{
	while(!(UART_SR & UART_SR_TXE));

	UART_DR = c;
}
#endif

